'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var reactDnd = require('react-dnd');
var HTML5Backend = _interopDefault(require('react-dnd-html5-backend'));
var emotion = require('emotion');
var themes = require('@anylate/themes');
var throttle = _interopDefault(require('lodash/throttle'));

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _taggedTemplateLiteral(strings, raw) {
  if (!raw) {
    raw = strings.slice(0);
  }

  return Object.freeze(Object.defineProperties(strings, {
    raw: {
      value: Object.freeze(raw)
    }
  }));
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) {
    return;
  }

  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var Context = React__default.createContext({});

var reducer = function reducer(state, action) {
  switch (action.type) {
    case 'setHover':
      console.log('setHover fired');
      return _objectSpread2({}, state, {
        hoverKey: action.payload
      });

    case 'setDragKey':
      return _objectSpread2({}, state, {
        dragKey: action.payload
      });

    case 'clean':
      return {
        hoverKey: null,
        dragKey: null
      };

    default:
      console.warn('NONE REDUCER HITTED!');
      return _objectSpread2({}, state);
  }
};

var Container = function Container(props) {
  var _useReducer = React.useReducer(reducer, {
    hoverKey: null,
    dragKey: null
  }),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  React.useEffect(function () {
    console.log('init container');
    return function () {
      console.log('clean container');
    };
  }, []);
  return React__default.createElement("div", {
    className: "container"
  }, React__default.createElement(Context.Provider, {
    value: {
      state: state,
      dispatch: dispatch
    }
  }, React__default.createElement(reactDnd.DndProvider, {
    backend: HTML5Backend
  }, props.children)));
};

var Card = function Card(_ref) {
  var id = _ref.id,
      children = _ref.children,
      dispatch = _ref.dispatch,
      index = _ref.index,
      nodeKey = _ref.nodeKey,
      editor = _ref.editor,
      state = _ref.state;
  var hoverKey = state.hoverKey,
      dragKey = state.dragKey;
  var moveCard = React.useCallback(function (dragKey, hoverKey) {
    var parentKey = editor.value.document.key;
    if (dragKey === hoverKey) return;
    var hoverIndex = editor.value.document.nodes.findIndex(function (i) {
      return i.key === "".concat(hoverKey);
    }); // hoverKey > dragKey ? hoverKey - 1 : hoverKey

    return editor.moveNodeByKey(dragKey, parentKey, hoverIndex);
  }, [editor]);

  var _useDrag = reactDnd.useDrag({
    item: {
      type: 'card',
      id: id,
      index: index,
      nodeKey: nodeKey
    },
    collect: function collect(monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    },
    options: {
      dropEffect: 'copy' // default is move

    },
    begin: function begin(item) {
      editor.blur().deselect();
      dispatch({
        type: 'setDragKey',
        payload: nodeKey
      });
    },
    end: function end(dropResult, monitor) {
      try {
        if (dropResult) {
          var _dragKey = dropResult.nodeKey;
          moveCard(_dragKey, hoverKey);
          setTimeout(function () {
            dispatch({
              type: 'clean'
            });
          }, 55);
        }
      } catch (e) {
        console.log('move end ,something seems to be wrong');
      }
    }
  }),
      _useDrag2 = _slicedToArray(_useDrag, 3),
      isDragging = _useDrag2[0].isDragging,
      drag = _useDrag2[1],
      preview = _useDrag2[2];

  var opacity = isDragging ? 0 : 1;

  var borderExtra = function borderExtra(drag, hover) {
    if (hover !== nodeKey) {
      return {};
    }

    var dragIndex = editor.value.document.nodes.findIndex(function (i) {
      return i.key === "".concat(drag);
    });
    var hoverIndex = editor.value.document.nodes.findIndex(function (i) {
      return i.key === "".concat(hover);
    });

    if (dragIndex > hoverIndex) {
      return {
        borderTopColor: 'green'
      };
    } else {
      return {
        borderBottomColor: 'green'
      };
    }
  };

  return React__default.createElement("div", {
    ref: preview,
    style: _objectSpread2({
      opacity: opacity,
      border: hoverKey === nodeKey ? '2px solid var(--theme-color-primary,#eee)' : '2px solid transparent'
    }, borderExtra(dragKey, hoverKey)),
    className: themes.componentClassName.DragContainer
  }, React__default.createElement("span", {
    ref: drag,
    className: emotion.cx('iconfont', 'icon-drag', themes.componentClassName.DragHandler),
    contentEditable: false
  }), children);
};

function _templateObject() {
  var data = _taggedTemplateLiteral(["\n          padding: 0;\n          margin: 0;\n        "]);

  _templateObject = function _templateObject() {
    return data;
  };

  return data;
}
var th = throttle(function (item, monitor, node, dispatch) {
  console.log('before setHover', item, monitor, node.key);
  var nodeKey = item.nodeKey;

  if (nodeKey !== node.key) {
    dispatch({
      type: 'setHover',
      payload: node.key
    });
  }
}, 50);

var CardHOC = function CardHOC(_ref) {
  var node = _ref.node,
      editor = _ref.editor,
      state = _ref.state,
      dispatch = _ref.dispatch,
      children = _ref.children,
      props = _objectWithoutProperties(_ref, ["node", "editor", "state", "dispatch", "children"]);

  var _useDrop = reactDnd.useDrop({
    accept: 'card',
    hover: function hover(item, monitor) {
      th(item, monitor, node, dispatch);
    },
    drop: function drop(dropResult, monitor) {
      console.log('drop!!!!!! enddrop', dropResult);
    }
  }),
      _useDrop2 = _slicedToArray(_useDrop, 2),
      drop = _useDrop2[1];

  return React__default.createElement(Card, _extends({}, props, {
    key: node.key,
    index: node.key,
    nodeKey: node.key,
    editor: editor,
    id: node.key,
    state: state,
    dispatch: dispatch
  }), React__default.createElement("figure", {
    className: emotion.cx(emotion.css(_templateObject())),
    ref: drop
  }, children));
};

var DraggablePlugin = function DraggablePlugin() {
  return {
    renderEditor: function renderEditor(props, editor, next) {
      return React__default.createElement(Container, _extends({}, props, {
        editor: editor
      }), next());
    },
    renderBlock: function renderBlock(props, editor, next) {
      var node = props.node;
      var document = editor.value.document;
      var children = next();

      if (document.nodes.includes(node)) {
        return React__default.createElement(Context.Consumer, null, function (_ref) {
          var state = _ref.state,
              dispatch = _ref.dispatch;
          return React__default.createElement(CardHOC, {
            key: "hoc-".concat(props.node.key),
            node: props.node,
            state: state,
            dispatch: dispatch,
            editor: editor
          }, children);
        });
      } else return React__default.createElement(React__default.Fragment, null, children);
    },
    onDrop: function onDrop(event, editor, next) {
      // event.preventDefault();
      return false;
    }
  };
};

exports.default = DraggablePlugin;
//# sourceMappingURL=draggable.js.map
